<?php
/**
 * Created by PhpStorm.
 * User: VITAS
 * Date: 09.03.2016
 * Time: 18:40
 */

// src/AppBundle/Entity/Mods.php

namespace AppBundle\Entity;

use Gedmo\Mapping\Annotation as Gedmo; // gedmo annotations
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity
 * @ORM\Table(name="mods")
 */
class Mods
{
    /**
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @Gedmo\Timestampable(on="create")
     * @ORM\Column(type="datetime")
     */
    private $created;

    /**
     * @ORM\ManyToMany(targetEntity="AppBundle\Entity\User")
     * @ORM\JoinTable(name="mods_users",
     *      joinColumns={@ORM\JoinColumn(name="mod_id", referencedColumnName="id")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="user_id", referencedColumnName="id")}
     * )
     */
    protected $owner;

    /**
     * @ORM\ManyToMany(targetEntity="AppBundle\Entity\User")
     * @ORM\JoinTable(name="mods_followers",
     *      joinColumns={@ORM\JoinColumn(name="mod_id", referencedColumnName="id")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="user_id", referencedColumnName="id")}
     * )
     */
    protected $followers;

    /**
     * @ORM\ManyToMany(targetEntity="AppBundle\Entity\Gversions")
     * @ORM\JoinTable(name="mods_gversions",
     *      joinColumns={@ORM\JoinColumn(name="game_id", referencedColumnName="id")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="gversion_id", referencedColumnName="id")}
     * )
     */
    protected $gversion;


    /**
     * @ORM\Column(type="string", length=100)
     */
    protected $mname;

    /**
     * @ORM\Column(type="string", length=100)
     * @Assert\NotBlank()
     */
    protected $package;

    /**
     * @ORM\Column(type="text")
     * @Assert\NotBlank()
     */
    protected $description;

    /**
     * @ORM\Column(type="string", length=100)
     */
    protected $mbanner;

    /**
     * @ORM\ManyToMany(targetEntity="AppBundle\Entity\Reviews")
     * @ORM\JoinTable(name="mods_reviews",
     *      joinColumns={@ORM\JoinColumn(name="mod_id", referencedColumnName="id")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="review_id", referencedColumnName="id")}
     * )
     */
    protected $reviews;

    /**
     * @ORM\Column(type="string", length=100)
     * @Assert\NotBlank()
     */
    protected $mversion;

    /**
     * @return mixed
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param mixed $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return mixed
     */
    public function getCreated()
    {
        return $this->created;
    }

    /**
     * @param mixed $created
     */
    public function setCreated($created)
    {
        $this->created = $created;
    }

    /**
     * @return mixed
     */
    public function getOwner()
    {
        return $this->owner;
    }

    /**
     * @param mixed $owner
     */
    public function setOwner($owner)
    {
        $this->owner = $owner;
    }

    /**
     * @return mixed
     */
    public function getFollowers()
    {
        return $this->followers;
    }

    /**
     * @param mixed $followers
     */
    public function setFollowers($followers)
    {
        $this->followers = $followers;
    }

    /**
     * @return mixed
     */
    public function getGversion()
    {
        return $this->gversion;
    }

    /**
     * @param mixed $gversion
     */
    public function setGversion($gversion)
    {
        $this->gversion = $gversion;
    }

    /**
     * @return mixed
     */
    public function getMname()
    {
        return $this->mname;
    }

    /**
     * @param mixed $mname
     */
    public function setMname($mname)
    {
        $this->mname = $mname;
    }

    /**
     * @return mixed
     */
    public function getPackage()
    {
        return $this->package;
    }

    /**
     * @param mixed $package
     */
    public function setPackage($package)
    {
        $this->package = $package;
    }

    /**
     * @return mixed
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param mixed $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return mixed
     */
    public function getMbanner()
    {
        return $this->mbanner;
    }

    /**
     * @param mixed $mbanner
     */
    public function setMbanner($mbanner)
    {
        $this->mbanner = $mbanner;
    }

    /**
     * @return mixed
     */
    public function getReviews()
    {
        return $this->reviews;
    }

    /**
     * @param mixed $reviews
     */
    public function setReviews($reviews)
    {
        $this->reviews = $reviews;
    }

    /**
     * @return mixed
     */
    public function getMversion()
    {
        return $this->mversion;
    }

    /**
     * @param mixed $mversion
     */
    public function setMversion($mversion)
    {
        $this->mversion = $mversion;
    }



    /**
     * Constructor
     */
    public function __construct()
    {
        $this->owner = new \Doctrine\Common\Collections\ArrayCollection();
        $this->followers = new \Doctrine\Common\Collections\ArrayCollection();
        $this->gversion = new \Doctrine\Common\Collections\ArrayCollection();
        $this->reviews = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Add owner
     *
     * @param \AppBundle\Entity\User $owner
     *
     * @return Mods
     */
    public function addOwner(\AppBundle\Entity\User $owner)
    {
        $this->owner[] = $owner;

        return $this;
    }

    /**
     * Remove owner
     *
     * @param \AppBundle\Entity\User $owner
     */
    public function removeOwner(\AppBundle\Entity\User $owner)
    {
        $this->owner->removeElement($owner);
    }

    /**
     * Add follower
     *
     * @param \AppBundle\Entity\User $follower
     *
     * @return Mods
     */
    public function addFollower(\AppBundle\Entity\User $follower)
    {
        $this->followers[] = $follower;

        return $this;
    }

    /**
     * Remove follower
     *
     * @param \AppBundle\Entity\User $follower
     */
    public function removeFollower(\AppBundle\Entity\User $follower)
    {
        $this->followers->removeElement($follower);
    }

    /**
     * Add gversion
     *
     * @param \AppBundle\Entity\Gversions $gversion
     *
     * @return Mods
     */
    public function addGversion(\AppBundle\Entity\Gversions $gversion)
    {
        $this->gversion[] = $gversion;

        return $this;
    }

    /**
     * Remove gversion
     *
     * @param \AppBundle\Entity\Gversions $gversion
     */
    public function removeGversion(\AppBundle\Entity\Gversions $gversion)
    {
        $this->gversion->removeElement($gversion);
    }

    /**
     * Add review
     *
     * @param \AppBundle\Entity\Reviews $review
     *
     * @return Mods
     */
    public function addReview(\AppBundle\Entity\Reviews $review)
    {
        $this->reviews[] = $review;

        return $this;
    }

    /**
     * Remove review
     *
     * @param \AppBundle\Entity\Reviews $review
     */
    public function removeReview(\AppBundle\Entity\Reviews $review)
    {
        $this->reviews->removeElement($review);
    }

    /**
     * Set mbanners
     *
     * @param string $mbanners
     *
     * @return Mods
     */
    public function setMbanners($mbanners)
    {
        $this->mbanners = $mbanners;

        return $this;
    }

    /**
     * Get mbanners
     *
     * @return string
     */
    public function getMbanners()
    {
        return $this->mbanners;
    }
}
